﻿#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Audio;
#endregion

namespace LD27
{
    public class AnimatedTexture
    {
        public AnimatedTexture(GraphicsDevice device, Texture2D texture, int frameWidth, TimeSpan frameDisplayDuration)
        {
            Texture = texture;
            FrameWidth = frameWidth;
            FrameDisplayDuration = frameDisplayDuration;

            if (texture.Width % frameWidth != 0)
                throw new Exception();
            FrameCount = texture.Width / frameWidth;

            _spriteBatch = new SpriteBatch(device);
        }

        public TimeSpan? OddFrameDisplayDuration { get; set; }

        public bool IsLooped { get; set; }
        
        public TimeSpan FrameDisplayDuration { get; private set; }

        TimeSpan _lastFrameChangeTime;

        SpriteBatch _spriteBatch;

        public bool Update(GameTime gameTime)
        {
            if (!IsAnimationRunning)
                return true;

            if (!_drawnFirstFrame)
                return false;

            var duration = FrameDisplayDuration;

            if (OddFrameDisplayDuration != null && CurrentFrame % 2 != 0)
                duration = OddFrameDisplayDuration.Value;

            if (gameTime.TotalGameTime - _lastFrameChangeTime >= duration)
            {
                if (++CurrentFrame >= FrameCount)
                {
                    if (IsLooped)
                        CurrentFrame = 0;
                    else
                    {
                        IsAnimationRunning = false;
                        return true;
                    }
                }

                _lastFrameChangeTime = gameTime.TotalGameTime;
            }

            return false;
        }

        public Vector2 Position { get; set; }
        public Vector2 RotationOrigin { get; set; }
        public float RotationAngle { get; set; }
        public float Scale { get; set; }

        bool _drawnFirstFrame;

        public void Draw()
        {
            _drawnFirstFrame = true;

            _spriteBatch.Begin(SpriteSortMode.Deferred,
                BlendState.AlphaBlend,
                SamplerState.PointWrap,
                DepthStencilState.None, RasterizerState.CullCounterClockwise, null);


            _spriteBatch.Draw(Texture, Position,
                new Rectangle(CurrentFrame * FrameWidth, 0, FrameWidth, Texture.Height),
                Color.White, RotationAngle, RotationOrigin, Scale,
                SpriteEffects.None, 0);

            _spriteBatch.End();
        }

        bool _isAnimationRunning;

        public bool IsAnimationRunning
        {
            get
            {
                return _isAnimationRunning;
            }

            set
            {
                if (_isAnimationRunning == value)
                    return;

                _isAnimationRunning = value;

                _drawnFirstFrame = false;
            }
        }

        public int FrameCount { get; private set; }
        public int CurrentFrame { get; set; }

        public Texture2D Texture { get; private set; }
        public int FrameWidth { get; private set; }
    }
}
