﻿#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Audio;
#endregion

namespace LD27
{
    public class Asteroid : Destructable
    {
        public Asteroid(Level level, string textureName, string explosionTextureName, string explosionSoundName, float scale)
            : base(level, explosionTextureName, explosionSoundName, scale)
        {
            TextureName = textureName;

            AddToComponents();
        }

        public string TextureName { get; private set; }

        Texture2D _texture;

        protected override void LoadContent()
        {
            base.LoadContent();

            _texture = Game.Content.Load<Texture2D>(TextureName);

            Size = new Vector2(_texture.Width, _texture.Height);

            _spriteBatch = new SpriteBatch(GraphicsDevice);

            RotationAngle = Level.Random.Next(0, 360);
        }

        public override void Initialize()
        {
            base.Initialize();
        }


        public override void Update(GameTime gameTime)
        {
            base.Update(gameTime);
            
            float elapsed = (float)gameTime.ElapsedGameTime.TotalSeconds; 

            RotationAngle += elapsed;
            RotationAngle = RotationAngle % MathHelper.TwoPi;

        }

        SpriteBatch _spriteBatch;

        public override void Draw(GameTime gameTime)
        {
            base.Draw(gameTime);

            if (!Destroying)
            {
                _spriteBatch.Begin(SpriteSortMode.Deferred,
                    BlendState.AlphaBlend,
                    SamplerState.PointWrap,
                    DepthStencilState.None, RasterizerState.CullCounterClockwise, null);


                _spriteBatch.Draw(_texture, ViewportPosition, null, Color.White, RotationAngle, RotationOrigin, Scale,
                    SpriteEffects.None, 0);

                _spriteBatch.End();
            }
        }

    }
}
