﻿#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Audio;
#endregion

namespace LD27
{
    public class Boss : Destructable
    {
        public Boss(Level level, string textureName)
            : base(level, "Explosion", "ShipExplosionSound", 1)
        {
            CalculateOrbitAngle = true;

            TextureName = textureName;

            BulletCount = 3;

            AddToComponents();
        }

        public string TextureName { get; set; }

        Texture2D _texture;

        protected override void LoadContent()
        {
            base.LoadContent();

            _texture = Game.Content.Load<Texture2D>(TextureName);

            Size = new Vector2(_texture.Width, _texture.Height);

            _spriteBatch = new SpriteBatch(GraphicsDevice);
        }

        public override void Initialize()
        {
            base.Initialize();

            _lastKeyState = Keyboard.GetState();
        }

        KeyboardState _lastKeyState;


        Vector2? _velocityBeforeActivation;

        public int BulletCount { get; set; }

        public float ShotStrenght { get; set; }


        public override void Update(GameTime gameTime)
        {
            base.Update(gameTime);


            var ship = Level.Ship;

            var cv = ship.ViewportPosition - ViewportPosition;

            if (cv.Length() < Level.ViewportSizeCartesian.Y)
            {
                var vec = ship.PositionPolar - PositionPolar;
                vec.Normalize();


                if (_velocityBeforeActivation == null)
                {
                    for (int i = 0; i < BulletCount; i++)
                    {
                        var bullet = new Bullet(Level, "BulletEnemy");

                        bullet.PositionPolar = PositionPolar;
                        bullet.VelocityPolar = vec + new Vector2(50 * i * (i % 2 == 0 ? -1 : 1), -ShotStrenght);
                        
                        bullet = new Bullet(Level, "BulletEnemy");

                        bullet.PositionPolar = PositionPolar;
                        bullet.VelocityPolar = vec + new Vector2(50 * i * (i % 2 == 0 ? -1 : 1), ShotStrenght);
                    }
                }

                if (_velocityBeforeActivation == null)
                    _velocityBeforeActivation = VelocityPolar;

                VelocityPolar = new Vector2(150 * vec.X, 5 * vec.Y);
            }
            else
            {
                if(_velocityBeforeActivation != null)
                    VelocityPolar = _velocityBeforeActivation.Value;
                _velocityBeforeActivation = null;
            }
        }

        SpriteBatch _spriteBatch;

        public override void Draw(GameTime gameTime)
        {
            base.Draw(gameTime);

            _spriteBatch.Begin(SpriteSortMode.Deferred,
                BlendState.AlphaBlend,
                SamplerState.PointWrap,
                DepthStencilState.None, RasterizerState.CullCounterClockwise, null);

            if (!Destroying)
            {
                _spriteBatch.Draw(_texture, ViewportPosition, null, Color.White, RotationAngle, RotationOrigin, Scale,
                    SpriteEffects.FlipVertically, 0);
            }

            _spriteBatch.End();
        }
    }
}
