﻿#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Audio;
#endregion

namespace LD27
{
    public class Bullet : Destructable
    {
        public Bullet(Level level, string textureName)
            : base(level, null, null, 1)
        {
            CalculateOrbitAngle = true;
            SkipCollisionCheck = true;


            TextureName = textureName;

            AddToComponents();
        }

        TimeSpan? _birthTime;

        public bool IsAntiEnemy { get; set; }

        public string TextureName { get; set; }

        Texture2D _texture;

        protected override void LoadContent()
        {
            base.LoadContent();

            _texture = Game.Content.Load<Texture2D>(TextureName);

            Size = new Vector2(_texture.Width, _texture.Height);

            _spriteBatch = new SpriteBatch(GraphicsDevice);
        }

        public override void Initialize()
        {
            base.Initialize();
        }

        public override void Update(GameTime gameTime)
        {
            base.Update(gameTime);

            if (IsAntiEnemy && !Level.IsVisible(this))
            {
                Destroy(gameTime);
            }
            else
            {
                if (_birthTime == null)
                    _birthTime = gameTime.TotalGameTime;
                else
                {
                    if ((gameTime.TotalGameTime - _birthTime) > TimeSpan.FromMilliseconds(5000))
                        Destroy(gameTime);
                }
            }

        }

        SpriteBatch _spriteBatch;

        public override void Draw(GameTime gameTime)
        {
            base.Draw(gameTime);

            _spriteBatch.Begin(SpriteSortMode.Deferred,
                BlendState.AlphaBlend,
                SamplerState.PointWrap,
                DepthStencilState.None, RasterizerState.CullCounterClockwise, null);


            _spriteBatch.Draw(_texture, ViewportPosition, null, Color.White, RotationAngle, RotationOrigin, Scale,
                SpriteEffects.FlipVertically, 0);

            _spriteBatch.End();
        }
    }
}
