﻿#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.GamerServices;
#endregion

namespace LD27
{
    public class Camera : GameComponent
    {
        //Camera matrices
        public Matrix View { get; protected set; }
        public Matrix Projection { get; protected set; }

        // Camera vectors
        public Vector3 Position3D { get; protected set; }

        public Vector2 Position2D
        {
            get
            {
                return new Vector2(Position3D.X, -Position3D.Z);
            }
        }

        public Vector2 Direction2D
        {
            get
            {
                return new Vector2(Direction3D.X, -Direction3D.Z);
            }
        }

        public Vector3 Direction3D { get; private set; }

        public Vector3 Up { get; private set; }

        // Mouse stuff
        MouseState prevMouseState;

        // Max yaw/pitch variables
        float currentYaw = 0;

        float totalPitch = MathHelper.PiOver4;
        float currentPitch = 0;

        public Camera(Game game, Vector3 pos, Vector3 target, Vector3 up)
            : base(game)
        {
            // Build camera view matrix
            Position3D = pos;
            
            var direction3D = target - pos;
            direction3D.Normalize();
            Direction3D = direction3D;

            Up = up;
            CreateLookAt();


            Projection = Matrix.CreateOrthographic(
                Game.Window.ClientBounds.Width,
                Game.Window.ClientBounds.Height,
                0.1f, 20000);
        }

        public override void Initialize()
        {
            ResetMousePosition();

            base.Initialize();
        }

        void ResetMousePosition()
        {
            Mouse.SetPosition(Game.Window.ClientBounds.Width / 2,
                Game.Window.ClientBounds.Height / 2);

            prevMouseState = Mouse.GetState();
        }

        // Speed
        float speed = 0.015f;

        void CheckKeyboard()
        {
            Vector3 position = Position3D;
            int movementsRegistered = 0;

            bool up = Keyboard.GetState().IsKeyDown(Keys.Up);
            bool down = Keyboard.GetState().IsKeyDown(Keys.Down);
            bool left = Keyboard.GetState().IsKeyDown(Keys.Left);
            bool right = Keyboard.GetState().IsKeyDown(Keys.Right);


            if (up)
                movementsRegistered++;
            if (down)
                movementsRegistered++;
            if (left)
                movementsRegistered++;
            if (right)
                movementsRegistered++;

            var actualSpeed = speed / movementsRegistered;

            var direction = Direction3D;
            direction.Y = 0;

            if (Keyboard.GetState().IsKeyDown(Keys.Up))
            {
                position = position + direction * actualSpeed;
            }

            if (Keyboard.GetState().IsKeyDown(Keys.Down))
            {
                position = position - direction * actualSpeed;
            }

            const float strafeMultiplier = 0.8f;

            if (Keyboard.GetState().IsKeyDown(Keys.Left))
            {
                position = position + Vector3.Cross(Up, direction) * actualSpeed * strafeMultiplier;
            }

            if (Keyboard.GetState().IsKeyDown(Keys.Right))
            {
                position = position - Vector3.Cross(Up, direction) * actualSpeed * strafeMultiplier;
            }

            if (movementsRegistered > 0)
            {
                var movementDirection3D = position - Position3D;
                movementDirection3D.Normalize();

                var movementDirection2D = new Vector2(movementDirection3D.X, -movementDirection3D.Z);

                var position2d = new Vector2(Position3D.X, -Position3D.Z);

                var newPosition2d = position2d + movementDirection2D * 0.1f;

                    Position3D = position;
            }
        }

        void CheckMouse()
        {
            float m = 0.6f;

            // Yaw
            float yawAngle = (-MathHelper.PiOver4 / 320) *
                    (Mouse.GetState().X - prevMouseState.X) * m;

            Direction3D = Vector3.Transform(Direction3D,
                Matrix.CreateFromAxisAngle(Up, yawAngle));
            currentYaw += yawAngle;

            if (currentYaw > MathHelper.TwoPi)
                currentYaw /= MathHelper.TwoPi;

            // Pitch
            float pitchAngle = (MathHelper.PiOver4 / 320) *
                (Mouse.GetState().Y - prevMouseState.Y) * m;

            if (Math.Abs(currentPitch + pitchAngle) < totalPitch)
            {
                Direction3D = Vector3.Transform(Direction3D,
                    Matrix.CreateFromAxisAngle(
                        Vector3.Cross(Up, Direction3D), pitchAngle)); 
                currentPitch += pitchAngle;
            }

            ResetMousePosition();

            prevMouseState = Mouse.GetState();
        }

        public override void Update(GameTime gameTime)
        {
            //CheckKeyboard();

            //CheckMouse();

            CreateLookAt();

            base.Update(gameTime);
        }

        private void CreateLookAt()
        {
            View = Matrix.CreateLookAt(Position3D,
                Position3D + Direction3D, Up);
        }
    }
}
