﻿#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Audio;
#endregion

namespace LD27
{
    public class Destructable : Orbital
    {
        public Destructable(Level level, string explosionTextureName, string explosionSoundName, float scale)
            : base(level, true, scale)
        {
            ExplosionSoundName = explosionSoundName;
            ExplosionTextureName = explosionTextureName;

            Level.Destructables.Add(this);
        }

        public string ExplosionSoundName { get; private set; }

        public string ExplosionTextureName { get; private set; }

        AnimatedTexture _texture;

        protected override void LoadContent()
        {
            base.LoadContent();

            if(ExplosionSoundName != null)
                _explosionSound = Game.Content.Load<SoundEffect>(ExplosionSoundName);
            
            if(ExplosionTextureName != null)
                _texture = new AnimatedTexture(GraphicsDevice,
                    Game.Content.Load<Texture2D>(ExplosionTextureName),
                    64, TimeSpan.FromMilliseconds(70));

            _spriteBatch = new SpriteBatch(GraphicsDevice);
        }

        public bool SkipCollisionCheck { get; set; }

        void CheckForCollisions(GameTime gameTime)
        {
            if (!Level.IsInRangeForCollisionDetection(this) || SkipCollisionCheck)
                return;

            var hit = Level.CheckForCollisions(this, Level.Interactables);

            if (hit != null)
            {
                if (hit is Bullet)
                {
                    var b = (Bullet)hit;

                    if (b.IsAntiEnemy && !(this is Ship))
                    {
                        b.Destroy(gameTime);
                        this.Destroy(gameTime);
                    }
                }
            }
        }


        SoundEffect _explosionSound;


        public override void Initialize()
        {
            base.Initialize();
        }

        public override void Update(GameTime gameTime)
        {
            if (!Destroying)
            {
                CheckForCollisions(gameTime);
                base.Update(gameTime);
            }
            else
            {
                if (_texture == null || _texture.Update(gameTime))
                {
                    Level.Destructables.Remove(this);
                    Level.Interactables.Remove(this);
                    Game.Components.Remove(this);

                    IsDead = true;
                }
            }
        }

        SpriteBatch _spriteBatch;

        public override void Draw(GameTime gameTime)
        {
            base.Draw(gameTime);

            if (Destroying && _texture != null)
            {
                _texture.Position = ViewportPosition;
                _texture.RotationAngle = RotationAngle;
                _texture.RotationOrigin = RotationOrigin;
                _texture.Scale = Scale;

                _texture.Draw();

            }
        }

        public bool Destroying { get; private set; }

        TimeSpan _shotStartTime;

        public bool IsDead { get; set; }

        public void Destroy(GameTime gameTime)
        {
            Destroying = true;

            if(_texture != null)
                _texture.IsAnimationRunning = true;

            _shotStartTime = gameTime.TotalGameTime;

            if (_explosionSound != null)
                _explosionSound.Play();

            Level.OnObjectDestroyed(this);

            OnDestroyed();
        }

        protected virtual void OnDestroyed()
        {
        }

    }
}
