﻿#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Audio;
#endregion

namespace LD27
{
    public static class EnemyTextureGenerator
    {
        static bool _loaded;

        static GeneratedTexturePartsAtlas _hulls;
        static GeneratedTexturePartsAtlas _wings;

        static void Load(Level level)
        {
            _hulls = new GeneratedTexturePartsAtlas(
                level.Game.Content.Load<Texture2D>("Hulls"), 64);

            _wings = new GeneratedTexturePartsAtlas(
                level.Game.Content.Load<Texture2D>("Wings"), 32);
        }

        static Random _random = new Random(Environment.TickCount);


        public static GeneratedTexture Generate(Level level)
        {
            if (!_loaded)
                Load(level);

            var device = level.Game.GraphicsDevice;

            var texture = new GeneratedTexture(device);

            var hull = new GeneratedTexturePart(device,
                _hulls.Texture,
                _hulls.FrameWidth,
                _random.Next(_hulls.FrameCount),
                false, false);

            texture.Size = new Vector2(_hulls.FrameWidth, _hulls.Texture.Height);

            texture.AddPart(hull);

            float wingY = _random.Next(10);
            float wingX = _random.Next(5, 10);

            var leftWing = new GeneratedTexturePart(device,
                _wings.Texture,
                _wings.FrameWidth,
                _random.Next(_wings.FrameCount),
                false, false);
            leftWing.Position = new Vector2(wingX - _wings.FrameWidth, wingY);
            texture.AddPart(leftWing);

            var rightWing = new GeneratedTexturePart(device,
                _wings.Texture,
                _wings.FrameWidth,
                _random.Next(_wings.FrameCount),
                false, true);
            rightWing.Position = new Vector2(texture.Size.X - wingX, wingY);
            texture.AddPart(rightWing);

            return texture;
        }
    }
}
