﻿#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.GamerServices;
#endregion

namespace LD27
{
    public class Hud : DrawableGameComponent
    {
        public Hud(Level level)
            : base(level.Game)
        {
            Level = level;
        }

        SpriteBatch _spriteBatch;

        protected void AddToComponents()
        {
            Game.Components.Add(this);
        }

        Texture2D _lifeTexture;
        Texture2D _secondEmptyTexture;
        Texture2D _secondFullTexture;

        protected override void LoadContent()
        {
            base.LoadContent();

            _lifeTexture = Game.Content.Load<Texture2D>("Life");
            _secondEmptyTexture = Game.Content.Load<Texture2D>("SecondEmpty");
            _secondFullTexture = Game.Content.Load<Texture2D>("SecondFull");

            _spriteBatch = new SpriteBatch(GraphicsDevice);
        }

        public override void Initialize()
        {
            base.Initialize();
        }

        public Level Level { get; private set; }

        public override void Draw(GameTime gameTime)
        {
            base.Draw(gameTime);

            _spriteBatch.Begin(SpriteSortMode.Deferred,
                BlendState.AlphaBlend,
                SamplerState.PointWrap,
                DepthStencilState.None, RasterizerState.CullCounterClockwise, null);

            float spacing = 5;
            for (int i = 0; i < Level.LivesLeft; i++)
            {
                _spriteBatch.Draw(_lifeTexture,
                    new Vector2(5 + i * (_lifeTexture.Width + spacing), 10),
                    null, Color.White, 0, Vector2.Zero, 1,
                    SpriteEffects.None, 0);

            }

            var c = Level.SecondsToCollect.Count + Level.CollectedSeconds.Count;

            for (int i = 0; i < c; i++)
            {
                _spriteBatch.Draw(i < Level.CollectedSeconds.Count ? _secondFullTexture : _secondEmptyTexture,
                    new Vector2(Level.ViewportSizeCartesian.X - 5 - (c - i) * (_secondFullTexture.Width + spacing), 10),
                    null, Color.White, 0, Vector2.Zero, 1,
                    SpriteEffects.None, 0);

            }

            _spriteBatch.End();
        }

    }
}
