﻿
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.GamerServices;
using System.Diagnostics;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Media;
using System.Runtime.InteropServices;

namespace LD27
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class LD27Game : Game
    {
        GraphicsDeviceManager graphics;

        public LD27Game()
            : base()
        {
            graphics = new GraphicsDeviceManager(this);
            
            Content.RootDirectory = "Content";

            Game = this;
        }

        protected override void Initialize()
        {
            _camera = new Camera(this, new Vector3(0, 0.3f, 0),
                new Vector3(0, 0.3f, 0), Vector3.Up);
            Components.Add(_camera);

            IsMouseVisible = false;
            
            graphics.SynchronizeWithVerticalRetrace = false;
            this.IsFixedTimeStep = false;


            base.Initialize();

            graphics.IsFullScreen = true;

            //graphics.IsFullScreen = false;

            graphics.PreferredBackBufferHeight = 1440;
            graphics.PreferredBackBufferWidth = 900;
            graphics.ApplyChanges();
        }

        void PlayNextLevel()
        {
            if (++_currentLevel > 3)
            {
                _youWonScreen = new TextScreen(this, "", Color.Blue);
                _youWonScreen.Scale = 1;
                _youWonScreen.Text = "You captured the nine species of rocketoids, but never found the last one,\n" +
                    "the one which gets born on the tenth second. You had to come back to Earth with only nine\n" +
                    "species. But then, one night behind the ship contol computer, you realized that...";

                Components.Add(_youWonScreen);

                return;
            }

            _levelLoadingScreen = new TextScreen(this, "", Color.Blue);
            _levelLoadingScreen.Text = "Orbit " + _currentLevel + "\nCapture 3 species";

            Components.Add(_levelLoadingScreen);
        }

        void ShowMainScreen()
        {
            _currentLevel = 0;

            _mainScreen = new MainScreen(this);
            Components.Add(_mainScreen);
        }

        int _currentLevel;

        Level _level;
        TextScreen _levelLoadingScreen;
        TextScreen _levelCompletedScreen;
        TextScreen _gameOverScreen;
        TextScreen _youWonScreen;
        TextScreen _tenthIsYouScreen;
        MainScreen _mainScreen;

        SpriteFont _monoFont;
        int _totalFrames = 0;
        float _elapsedTime = 0.0f;
        int _fps = 0;

        public static LD27Game Game { get; private set; }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            _spriteBatch = new SpriteBatch(GraphicsDevice);

            _monoFont = Content.Load<SpriteFont>("MonoFont");
        }

 
        Camera _camera;

        public Camera Camera
        {
            get { return _camera; }
        }

        SpriteBatch _spriteBatch;

        protected override void UnloadContent()
        {
        }

        void UpdateFpsCounter(GameTime gameTime)
        {
            _elapsedTime += (float)gameTime.ElapsedGameTime.TotalMilliseconds;

            if (_elapsedTime >= 1000.0f)
            {
                _fps = _totalFrames;
                _totalFrames = 0;
                _elapsedTime = 0;
            }
        }

        void StartLevel()
        {
            switch (_currentLevel)
            {
                case 1:
                    _level = new Level1(this);
                    break;
                case 2:
                    _level = new Level2(this);
                    break;
                case 3:
                    _level = new Level3(this);
                    break;
                default:
                    throw new Exception();
            }

            Components.Add(_level);

            _firstDraw = false;
        }

        protected override void Update(GameTime gameTime)
        {
            UpdateFpsCounter(gameTime);

            if (Keyboard.GetState().IsKeyDown(Keys.Escape))
                Exit();

            //if (_level == null)
            //{
            //    _currentLevel = 3;
            //    StartLevel();
            //}
            //base.Update(gameTime);
            //return;

            if (_levelLoadingScreen != null)
            {
                if (_levelLoadingScreen.IsFinished)
                {
                    Components.Remove(_levelLoadingScreen);
                    _levelLoadingScreen = null;

                    StartLevel();
                }
            }
            else if (_gameOverScreen != null)
            {
                if (_gameOverScreen.IsFinished)
                {
                    Components.Remove(_gameOverScreen);
                    _gameOverScreen = null;

                    ShowMainScreen();
                }
            }
            else if (_levelCompletedScreen != null)
            {
                if (_levelCompletedScreen.IsFinished)
                {
                    Components.Remove(_levelCompletedScreen);
                    _levelCompletedScreen = null;

                    PlayNextLevel();
                }
            }
            else if (_mainScreen != null)
            {
                if (_mainScreen.IsFinished)
                {
                    Components.Remove(_mainScreen);
                    _mainScreen = null;

                    PlayNextLevel();
                }
            }
            else if (_youWonScreen != null)
            {
                if (_youWonScreen.IsFinished)
                {
                    Components.Remove(_youWonScreen);
                    _youWonScreen = null;

                    _tenthIsYouScreen = new TextScreen(this, "", Color.Blue);
                    _tenthIsYouScreen.Scale = 3;
                    _tenthIsYouScreen.Text = "YOU ARE THE TENTH ROCKETOID.";

                    Components.Add(_tenthIsYouScreen);
                }
            }
            else if (_tenthIsYouScreen != null)
            {
                if (_tenthIsYouScreen.IsFinished)
                {
                    Exit();
                }
            }
            else if (_level != null)
            {
                if (_level.IsFailed)
                {
                    Components.Remove(_level);
                    _level.RemoveChildren();
                    _level = null;

                    _gameOverScreen = new TextScreen(this, "GAME OVER", Color.Red);
                    Components.Add(_gameOverScreen);
                }
                else if (_level.IsCompleted)
                {
                    Components.Remove(_level);
                    _level.RemoveChildren();
                    _level = null;

                    _levelCompletedScreen = new TextScreen(this, "Three species collected.", Color.Blue);
                    Components.Add(_levelCompletedScreen);
                }
            }
            else
            {
                ShowMainScreen();
            }

            base.Update(gameTime);
        }

        bool _firstDraw;

        protected override void Draw(GameTime gameTime)
        {
            if (_level != null && !_firstDraw)
            {
                _level.LoadLevel();
                _firstDraw = true;
            }

            GraphicsDevice.Clear(Color.Black);

            _totalFrames++;


            //_spriteBatch.Begin(SpriteSortMode.Immediate,
            //    BlendState.AlphaBlend,
            //    SamplerState.PointClamp,
            //    DepthStencilState.None, RasterizerState.CullCounterClockwise, null);

            //_spriteBatch.DrawString(_monoFont, string.Format("{0}", _fps),
            //    new Vector2(10.0f, 10.0f), Color.White);

            //_spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
