﻿#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.GamerServices;
using System.Collections;
using Microsoft.Xna.Framework.Audio;
#endregion

namespace LD27
{
    public abstract class Level : GameComponent
    {
        public Level(Game game)
            : base(game)
        {
            Destructables = new List<Destructable>();
            Interactables = new List<Orbital>();
            Seconds = new List<Second>();

            SecondsToCollect = new HashSet<int>();
            CollectedSeconds = new HashSet<int>();

            LivesLeft = 4;
        }


        public bool IsCompleted { get; set; }

        public bool IsFailed { get; set; }

        public int LivesLeft { get; set; }

        Vector2 _viewportPositionPolar;

        public Vector2 ViewportPositionPolar
        {
            get
            {
                return _viewportPositionPolar;
            }

            set
            {
                value = new Vector2(value.X, value.Y % 360);

                _viewportPositionPolar = value;

                View = GetViewMatrix(value);
            }
        }

        public Matrix GetViewMatrix(Vector2 polar)
        {
            var translation = Matrix.CreateTranslation(
                new Vector3(polar.X + ViewportSizeCartesian.X / 2, ViewportSizeCartesian.Y / 2, 0));

            var rotation = Matrix.CreateFromAxisAngle(-Vector3.UnitZ, MathHelper.ToRadians(polar.Y));

             return rotation * translation;
        }

        public virtual void OnObjectDestroyed(Destructable d)
        {
            //if (d is Guard)
            //{
            //    var enemy = new Guard(this, ((Guard)d).TextureName);
            //    enemy.VelocityPolar = new Vector2(0, d.VelocityPolar.Y);

            //    var r = Random.Next((int)(Radius - ViewportSizeCartesian.X / 2), (int)(Radius + ViewportSizeCartesian.X));
            //    enemy.PositionPolar =
            //        new Vector2(r, Random.Next(0, 360));
            //}
            //else if (d is Boss)
            //{
            //    var enemy = new Boss(this, ((Boss)d).TextureName);
            //    enemy.VelocityPolar = new Vector2(0, d.VelocityPolar.Y);

            //    var r = Random.Next((int)(Radius - ViewportSizeCartesian.X / 2), (int)(Radius + ViewportSizeCartesian.X));
            //    enemy.PositionPolar =
            //        new Vector2(r, Random.Next(0, 360));
            //}
        }

        public Matrix View { get; private set; }

        public Vector2 ViewportSizeCartesian { get; private set; }

        public override void Initialize()
        {
            base.Initialize();

            _lastKeyState = Keyboard.GetState();
        }

        public HashSet<int> SecondsToCollect { get; private set; }
        public HashSet<int> CollectedSeconds { get; private set; }

        public int Radius { get; private set; }

        protected virtual void OnLoadLevel()
        {

        }

        bool _loaded;

        public void LoadLevel()
        {
            Radius = 4000;

            var viewportWidth = GraphicsDevice.Viewport.Width;
            var viewportHeight = GraphicsDevice.Viewport.Height;

            ViewportSizeCartesian = new Vector2(viewportWidth, viewportHeight);

            ViewportPositionPolar = new Vector2(Radius, 0);

            ViewportVelocityPolar = new Vector2(0, 10);



            for (int i = 0; i < 1000; i++)
            {
                var x = Random.Next(0, 30);
                var scale = 1 + 1f / (x == 0 ? 1 : x);
                var stone = new BackgroundObject(this, "Star" + Random.Next(1, 4), scale);
                stone.VelocityPolar = new Vector2(0, 0);

                var r = Random.Next(Radius - viewportWidth / 2, Radius + viewportWidth / 2);
                stone.PositionPolar =
                    new Vector2(r, Random.Next(0, 360));
            }

            OnLoadLevel();

            CreateShip();

            _hud = new Hud(this);
            Game.Components.Add(_hud);


            _music = Game.Content.Load<SoundEffect>("Song");

            _musicInstance = _music.CreateInstance();
            _musicInstance.IsLooped = true;
            _musicInstance.Volume = 0.5f;
            _musicInstance.Play();

            _loaded = true;
        }

        SoundEffectInstance _musicInstance;
        SoundEffect _music;

        Hud _hud;

        void CreateShip()
        {
            if (--LivesLeft == 0)
            {
                IsFailed = true;
                return;
            }

            Ship = new Ship(this);
            Ship.PositionPolar = ViewportPositionPolar + new Vector2(0, 180);
        }

        public Ship Ship { get; private set; }

        public static Random Random = new Random(Environment.TickCount);

        public Vector2 ViewportVelocityPolar { get; set; }

        KeyboardState _lastKeyState;

        public override void Update(GameTime gameTime)
        {
            if (!_loaded)
                return;

            base.Update(gameTime);

            if (SecondsToCollect.Count == 0)
            {
                IsCompleted = true;
                return;
            }

            if (Ship != null && Ship.IsDead)
                CreateShip();

            var elapsed = gameTime.ElapsedGameTime;

            ViewportPositionPolar += (float)elapsed.TotalSeconds * ViewportVelocityPolar;


            HandleInput(gameTime);
        }


        public void RemoveChildren()
        {
            var toRemove = new List<GameComponent>();

            foreach (GameComponent c in Game.Components)
            {
                if (c is Orbital)
                    toRemove.Add(c);
            }

            foreach (var r in toRemove)
                Game.Components.Remove(r);

            Game.Components.Remove(_hud);

            _musicInstance.Stop();
        }

        public List<Destructable> Destructables { get; private set; }

        public List<Second> Seconds { get; private set; }

        public List<Orbital> Interactables { get; private set; }

        void HandleInput(GameTime gameTime)
        {
            if (Ship == null)
                return;

            var keyState = Keyboard.GetState();



            _lastKeyState = keyState;
        }

        public Orbital CheckForCollisions(Orbital obj, ICollection objects)
        {
            foreach (var o in objects)
            {
                var d = (Orbital)o;

                if (d == obj)
                    continue;

                var pos = d.ViewportPosition + d.Size / 2;

                if (!IsInRangeForCollisionDetection(d))
                    continue;

                var intersection = MathUtils.TestCircleIntersection(obj.ViewportPosition + obj.Size / 2,
                    obj.CollisionDetectionRadius,
                    pos, d.CollisionDetectionRadius);

                if (intersection)
                    return d;
            }

            return null;
        }

        public bool IsInRangeForCollisionDetection(Orbital o)
        {
            var pos = o.ViewportPosition + o.Size / 2;
            return new Rectangle(-(int)ViewportSizeCartesian.X, -(int)ViewportSizeCartesian.Y, (int)ViewportSizeCartesian.X * 3, (int)ViewportSizeCartesian.Y * 3).Contains(
                new Point((int)pos.X, (int)pos.Y));
        }

        public bool IsVisible(Orbital o)
        {
            var pos = o.ViewportPosition + o.Size / 2;
            return new Rectangle(0, 0, (int)ViewportSizeCartesian.X, (int)ViewportSizeCartesian.Y).Contains(
                new Point((int)pos.X, (int)pos.Y));
        }


        public Orbital CheckForRayIntersections(Orbital obj, Vector2 sight, ICollection objects, out float distance)
        {
            distance = float.MaxValue;
            Orbital hit = null;

            foreach (var o in objects)
            {
                var d = (Orbital)o;

                if (d == obj)
                    continue;

                var pos = d.ViewportPosition + d.Size / 2;

                if (!IsInRangeForCollisionDetection(d))
                    continue;

                var intersection = MathUtils.TestRayCircleIntersection(obj.ViewportPosition + obj.Size / 2, sight,
                    pos, d.CollisionDetectionRadius);

                if (intersection != null)
                {
                    if (distance > intersection.Value)
                    {
                        distance = intersection.Value;
                        hit = d;
                    }
                }
            }

            return hit;
        }

    }
}
