﻿#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.GamerServices;
using System.Collections;
#endregion

namespace LD27
{
    public class MainScreen : DrawableGameComponent
    {
        public MainScreen(Game game)
            : base(game)
        {
        }

        public override void Initialize()
        {
            base.Initialize();

            _previousKeyState = Keyboard.GetState();
        }

        protected override void LoadContent()
        {
            base.LoadContent();

            _spriteBatch = new SpriteBatch(GraphicsDevice);

            _titleTexture = Game.Content.Load<Texture2D>("Title");
            _monoFont = Game.Content.Load<SpriteFont>("MonoFont");
        }


        Texture2D _titleTexture;

        SpriteBatch _spriteBatch;

        SpriteFont _monoFont;

        public bool IsFinished { get; private set; }


        KeyboardState _previousKeyState;

        public override void Update(GameTime gameTime)
        {
            base.Update(gameTime);


            if (_previousKeyState.GetPressedKeys().Length == 0 &&
                Keyboard.GetState().GetPressedKeys().Length > 0)
                IsFinished = true;

            _previousKeyState = Keyboard.GetState();

        }


        public override void Draw(GameTime gameTime)
        {
            _spriteBatch.Begin(SpriteSortMode.Immediate,
                BlendState.AlphaBlend,
                SamplerState.PointClamp,
                DepthStencilState.None, RasterizerState.CullCounterClockwise, null);

            _spriteBatch.Draw(_titleTexture,
                new Vector2((GraphicsDevice.Viewport.Width - _titleTexture.Width) / 2, 10),
                null, Color.White, 0, Vector2.Zero, 1, SpriteEffects.None, 0);


            string str = "You are a commander of space biology exploration expedition, sent to a distant \n" +
                             "planetary system.\n\n" +
                             "The subject of your research is highly unusual -- a life form\n" +
                             "like no other known in the galaxy - the rocketoids. These creatures\n" +
                             "live in open space, constantly orbiting around the home planet.\n" +
                             "Presumably feeding from star radiation, communicating among themselves through\n" +
                             "radiowaves, these silent semiconductor space natives are unexplored\n" +
                             "by human yet. Your task is to capture several of them and bring to Earth for research.\n\n" +
                             "These creatures have interesting way of giving birth: each mother has exactly\n" +
                             "10 children and each of them gets born within exactly one second. Thus,\n" +
                             "all the children get born in exactly 10 seconds. Going further, each child\n" +
                             "born on each of consecutive 10 seconds is of completely different biological properties.\n" +
                             "You will need to capture all 10 \"species\" of the rocketoids.";

            var size = _monoFont.MeasureString(str);

            _spriteBatch.DrawString(_monoFont, str,
                new Vector2((GraphicsDevice.Viewport.Width - size.X) / 2, 10 + _titleTexture.Height + 20),
                Color.Cyan, 0, Vector2.Zero, 1, SpriteEffects.None, 0);
            
             str = "Arrow keys to move, Z to shoot, X to use tractor beam.";

            var size1 = _monoFont.MeasureString(str);

            _spriteBatch.DrawString(_monoFont, str,
                new Vector2((GraphicsDevice.Viewport.Width - size1.X) / 2, 10 + _titleTexture.Height + 20 + 20 + size.Y),
                Color.Red, 0, Vector2.Zero, 1, SpriteEffects.None, 0);

            str = "Made by Yevgeni Zolotko in 48 hours for LD27. See yez0.com for more games.";

            var size2 = _monoFont.MeasureString(str);

            _spriteBatch.DrawString(_monoFont, str,
                new Vector2((GraphicsDevice.Viewport.Width - size2.X) / 2, 10 + _titleTexture.Height + 20 + 20 + 20 + size.Y + size1.Y),
                Color.Gray, 0, Vector2.Zero, 1, SpriteEffects.None, 0);

            _spriteBatch.End();

            
            base.Draw(gameTime);
        }
    }
}
