﻿#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.GamerServices;
#endregion

namespace LD27
{
    public static class MathUtils
    {

        public static Vector2 PolarToCartesian(Vector2 polar)
        {
            var theta = MathHelper.ToRadians(polar.Y);

            return new Vector2(
                polar.X * (float)Math.Cos(theta),
                polar.X * (float)Math.Sin(theta));
        }

        public static bool SolveQuadraticEqation(float a, float b, float c, out float x1, out float x2)
        {
            double sqrtpart = b * b - 4 * a * c;
            if (sqrtpart > 0)
            {
                x1 = (-b + (float)System.Math.Sqrt(sqrtpart)) / (2 * a);
                x2 = (-b - (float)System.Math.Sqrt(sqrtpart)) / (2 * a);
                return true;
            }

            if (sqrtpart < 0)
            {
                x1 = x2 = 0;
                return false;
            }

            x1 = x2 = (-b + (float)System.Math.Sqrt(sqrtpart)) / (2 * a);
            return true;
        }


        public static float? TestRayCircleIntersection(Vector2 from, Vector2 direction, Vector2 center, float radius)
        {
            float a = Vector2.Dot(direction, direction);

            float b = 2 * Vector2.Dot(direction, from - center);

            float c = Vector2.Dot(from - center, from - center) - radius * radius;

            float x1, x2;

            if (!SolveQuadraticEqation(a, b, c, out x1, out x2))
                return null;

            float hitT = Math.Min(x1, x2);

            if (hitT < 0.00001)
                return null;

            return hitT;
        }

        public static bool TestCircleIntersection(Vector2 centerA, double radiusA, Vector2 centerB, double radiusB)
        {
            double dX = (double)(centerB.X - centerA.X);
            double dY = (double)(centerB.Y - centerA.Y);
            double d = Math.Sqrt(dX * dX + dY * dY);

            if (d > (radiusA + radiusB))
                return false;

            return true;
        }
    }
}
