﻿#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.GamerServices;
#endregion

namespace LD27
{
    public class Orbital : DrawableGameComponent
    {
        public Orbital(Level level, bool isInteractable, float scale)
            : base(level.Game)
        {
            Level = level;


            Scale = scale;

            if (isInteractable)
                Level.Interactables.Add(this);

            IsInteractable = isInteractable;
        }

        public float Scale { get; private set; }


        protected void AddToComponents()
        {
            Game.Components.Add(this);
        }

        public bool IsInteractable { get; private set; }

        protected override void LoadContent()
        {
            base.LoadContent();
        }

        public override void Initialize()
        {
            base.Initialize();
        }

        protected virtual bool CanMove(GameTime gameTime)
        {
            return true;
        }

        public override void Update(GameTime gameTime)
        {
            base.Update(gameTime);

            var elapsed = gameTime.ElapsedGameTime;

            //if (CanMove(gameTime))
            {
                PositionPolar += (float)elapsed.TotalSeconds * VelocityPolar;
            }
        }

        public Level Level { get; private set; }

        Vector2 _size;

        public Vector2 Size
        {
            get
            {
                return _size;
            }

            protected set
            {
                _size = value;

                CollisionDetectionRadius = Math.Min(value.X, value.Y) / 2 * Scale;
                RotationOrigin = _size * 0.5f;
            }
        }

        public float CollisionDetectionRadius { get; set; }

        public float RotationAngle { get; set; }

        public bool CalculateOrbitAngle { get; set; }

        Vector2 _positionPolar;

        public Vector2 PositionPolar
        {
            get
            {
                return _positionPolar;
            }

            set
            {
                value = new Vector2(value.X, value.Y % 360);

                _positionPolar = value;

                PositionCartesian = MathUtils.PolarToCartesian(value);

                var v = Vector3.Transform(new Vector3(PositionCartesian, 0), Level.View);

                ViewportPosition = new Vector2(v.X, v.Y);

                if(CalculateOrbitAngle)
                    CalculateRotationAngle();
            }
        }

        public Vector2 GetViewportPositionFromPolar(Vector2 polar)
        {
            PositionCartesian = MathUtils.PolarToCartesian(polar);

            var v = Vector3.Transform(new Vector3(PositionCartesian, 0), Level.View);

            return new Vector2(v.X, v.Y);
        }


        void CalculateRotationAngle()
        {
            var c = MathUtils.PolarToCartesian(new Vector2(PositionPolar.X, PositionPolar.Y - Level.ViewportPositionPolar.Y));

            c += Size / 2;

            var normal = new Vector3(c + Size / 2, 0);
            normal.Normalize();

            var tangent = Vector3.Cross(normal, Vector3.UnitZ);
            tangent.Normalize();

            var axis = Vector3.UnitY;

            RotationAngle = (float)GetSignedAngleBetween2DVectors(axis, tangent, -normal);
        }

        public Vector2 GetSightVector()
        {
            var c = MathUtils.PolarToCartesian(new Vector2(PositionPolar.X, PositionPolar.Y - Level.ViewportPositionPolar.Y));

            c += Size / 2;

            var normal = new Vector3(c + Size / 2, 0);
            normal.Normalize();

            var tangent = Vector3.Cross(normal, Vector3.UnitZ);
            tangent.Normalize();

            return -new Vector2(tangent.X, tangent.Y);
        }

        double GetSignedAngleBetween2DVectors(Vector3 FromVector, Vector3 DestVector, Vector3 DestVectorsRight)
        {
            FromVector.Normalize();
            DestVector.Normalize();
            DestVectorsRight.Normalize();

            float forwardDot = Vector3.Dot(FromVector, DestVector);
            float rightDot = Vector3.Dot(FromVector, DestVectorsRight);

            forwardDot = MathHelper.Clamp(forwardDot, -1.0f, 1.0f);

            double angleBetween = Math.Acos(forwardDot);

            if (rightDot < 0.0f)
                angleBetween *= -1.0f;

            return angleBetween;
        }

        public float UnsignedAngleBetweenTwoV3(Vector3 v1, Vector3 v2)
        {
            v1.Normalize();
            v2.Normalize();
            double Angle = (float)Math.Acos(Vector3.Dot(v1, v2));
            return (float)Angle;
        }

        public Vector2 CenterPositionCartesian
        {
            get
            {
                return PositionCartesian + Size / 2;
            }
        }

        public Vector2 PositionCartesian { get; private set; }

        public Vector2 ViewportPosition { get; private set; }

        public virtual Vector2 VelocityPolar { get; set; }

        public Vector2 RotationOrigin { get; private set; }

    }
}
