﻿#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Audio;
#endregion

namespace LD27
{
    public class Second : Destructable
    {
        public Second(Level level, int number)
            : base(level, null, "SecondCollectSound", 1)
        {
            CalculateOrbitAngle = true;
            Number = number;

            SkipCollisionCheck = true;

            Level.Seconds.Add(this);

            AddToComponents();
        }

        protected override void OnDestroyed()
        {
            base.OnDestroyed();

            Level.Seconds.Remove(this);
        }

        public int Number { get; set; }

        AnimatedTexture _texture;

        protected override void LoadContent()
        {
            base.LoadContent();

            _texture = new AnimatedTexture(
                GraphicsDevice,
                Game.Content.Load<Texture2D>("Second" + Number),
                64,
                TimeSpan.FromMilliseconds(1000));

            _texture.OddFrameDisplayDuration = TimeSpan.FromMilliseconds(200);

            _texture.IsLooped = true;
            _texture.IsAnimationRunning = true;

            Size = new Vector2(_texture.FrameWidth, _texture.Texture.Height);

            _spriteBatch = new SpriteBatch(GraphicsDevice);
        }

        public override void Initialize()
        {
            base.Initialize();

            _lastKeyState = Keyboard.GetState();
        }

        KeyboardState _lastKeyState;


        Vector2? _velocityBeforeActivation;

        public override void Update(GameTime gameTime)
        {
            base.Update(gameTime);


            if (IsDragged)
            {
                var ship = Level.Ship;

                var vec = ship.PositionPolar - PositionPolar;
                vec.Normalize();

                if (_velocityBeforeActivation == null)
                    _velocityBeforeActivation = VelocityPolar;

                VelocityPolar = new Vector2(150 * vec.X, 5 * vec.Y);
            }
            else
            {
                if (_velocityBeforeActivation != null)
                    VelocityPolar = _velocityBeforeActivation.Value;
                _velocityBeforeActivation = null;
            }

            _texture.Update(gameTime);
        }

        public bool IsDragged { get; set; }


        SpriteBatch _spriteBatch;

        public override void Draw(GameTime gameTime)
        {
            base.Draw(gameTime);

            _spriteBatch.Begin(SpriteSortMode.Deferred,
                BlendState.AlphaBlend,
                SamplerState.PointWrap,
                DepthStencilState.None, RasterizerState.CullCounterClockwise, null);


            _texture.Position = ViewportPosition;
            _texture.RotationAngle = RotationAngle;
            _texture.RotationOrigin = RotationOrigin;
            _texture.Scale = Scale;

            _texture.Draw();

            _spriteBatch.End();
        }
    }
}
