﻿#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Audio;
#endregion

namespace LD27
{
    public class Ship : Destructable
    {
        public Ship(Level level)
            : base(level, "Explosion", "ShipExplosionSound", 1)
        {
            CalculateOrbitAngle = true;

            CollisionDetectionRadius = 52;

            AddToComponents();
        }

        Texture2D _texture;

        Texture2D _laserTexture;

        SoundEffect _laserSound;
        SoundEffect _tractorBeamSound;

        protected override void LoadContent()
        {
            base.LoadContent();

            _texture = Game.Content.Load<Texture2D>("Ship");
            _laserTexture = Game.Content.Load<Texture2D>("TractorBeam");
            _laserSound = Game.Content.Load<SoundEffect>("LaserShoot");
            _tractorBeamSound = Game.Content.Load<SoundEffect>("TractorBeamSound");

            Size = new Vector2(_texture.Width, _texture.Height);

            _spriteBatch = new SpriteBatch(GraphicsDevice);
        }

        public override void Initialize()
        {
            base.Initialize();

            _lastKeyState = Keyboard.GetState();
        }


        KeyboardState _lastKeyState;



        bool IsOnTheEdge(GameTime gameTime)
        {
            var elapsed = gameTime.ElapsedGameTime;

            var nextPos = PositionPolar + (float)elapsed.TotalSeconds * VelocityPolar;

            var nextViewportPos = GetViewportPositionFromPolar(nextPos);

            if (nextViewportPos.X < 0 ||
               nextViewportPos.Y < 0 ||
               nextViewportPos.X - Size.X > Level.ViewportSizeCartesian.X ||
                nextViewportPos.Y - Size.Y > Level.ViewportSizeCartesian.Y)
                return true;

            return false;
        }

        void HandleInput(GameTime gameTime)
        {
            var keyState = Keyboard.GetState();
            if (keyState.GetPressedKeys().Length > 0)
            {
                float dx = 0;
                float dy = 0;

                float xv = 500;
                float yv = 10 * Level.ViewportPositionPolar.X / PositionPolar.X;

                bool horz = false;

                if (keyState.IsKeyDown(Keys.Left))
                {
                    dx = xv;
                    horz = true;
                }
                else if (keyState.IsKeyDown(Keys.Right))
                {
                    dx = -xv;
                    horz = true;
                }

                if (keyState.IsKeyDown(Keys.Down))
                    dy = -yv;
                else if (keyState.IsKeyDown(Keys.Up))
                    dy = yv;

                VelocityPolar = new Vector2(dx, dy);

                //var nextVelocity = new Vector2(dx, dy);

                //var nextPos = PositionPolar + (float)gameTime.ElapsedGameTime.TotalSeconds * nextVelocity;

                //var nextViewportPos = GetViewportPositionFromPolar(nextPos);


                //if (horz)
                //{
                //    if (!(nextViewportPos.X < 0 ||
                //        nextViewportPos.X > Level.ViewportSizeCartesian.X))
                //    {
                //        VelocityPolar = new Vector2(dx, dy);
                //    }
                //    else
                //        VelocityPolar = new Vector2(0, 0);
                //}
                //else
                //{
                //    if (!(nextViewportPos.Y < 0 ||
                //        nextViewportPos.Y > Level.ViewportSizeCartesian.Y))
                //    {
                //        VelocityPolar = new Vector2(dx, dy);
                //    }
                //    else
                //        VelocityPolar = new Vector2(0, 0);
                //}
            }
            else
                VelocityPolar = new Vector2(0, 0);

            if (keyState.IsKeyDown(Keys.Z) && !_lastKeyState.IsKeyDown(Keys.Z) &&
                (gameTime.TotalGameTime - _lastShotTime) > TimeSpan.FromMilliseconds(500))
            {
                var bullet = new Bullet(Level, "BulletShip");

                bullet.IsAntiEnemy = true;
                bullet.PositionPolar = PositionPolar;
                bullet.VelocityPolar = new Vector2(0, 20);

                _laserSound.Play();

                _lastShotTime = gameTime.TotalGameTime;
            }


            if (keyState.IsKeyDown(Keys.X) && !_lastKeyState.IsKeyDown(Keys.X))
            {
                _tractorBeamStartTime = gameTime.TotalGameTime;
                IsTractorBeamActive = true;
            }
            else if (!keyState.IsKeyDown(Keys.X) && _lastKeyState.IsKeyDown(Keys.X))
            {
                IsTractorBeamActive = false;
            }

            _lastKeyState = keyState;
        }

        TimeSpan? _birthTime;

        TimeSpan _lastShotTime;

        void CheckForCollisions(GameTime gameTime)
        {
            var hit = Level.CheckForCollisions(this, Level.Interactables);

            if (hit != null)
            {
                if (hit is Second && _draggingSecond == hit)
                {
                    IsTractorBeamActive = false;

                    var s = (Second)hit;

                    if (!Level.CollectedSeconds.Contains(s.Number))
                        Level.CollectedSeconds.Add(s.Number);

                    if (Level.SecondsToCollect.Contains(s.Number))
                        Level.SecondsToCollect.Remove(s.Number);

                    s.Destroy(gameTime);
                }
                else if (hit is Bullet && ((Bullet)hit).IsAntiEnemy)
                {
                }
                else
                {
                    if (!_invinsible)
                    {
                        if(hit is Guard || hit is Boss)
                            ((Destructable)hit).Destroy(gameTime);

                        this.Destroy(gameTime);
                    }
                }
            }
        }


        protected override bool CanMove(GameTime gameTime)
        {
            return !IsOnTheEdge(gameTime);
        }


        public override void Update(GameTime gameTime)
        {
            base.Update(gameTime);

            _invinsible = gameTime.TotalGameTime - _birthTime < TimeSpan.FromMilliseconds(2000);

            if (_invinsible)
            {
                if (gameTime.TotalGameTime - _lastInvincibilityBlinkTime > TimeSpan.FromMilliseconds(100))
                {
                    _lastInvincibilityBlinkTime = gameTime.TotalGameTime;
                    _invinsibilityBlinkState = !_invinsibilityBlinkState;
                }
            }


            if (_birthTime == null)
                _birthTime = gameTime.TotalGameTime;

            HandleInput(gameTime);

            CheckForCollisions(gameTime);

            if (_isTractorBeamActive)
                UpdateTractorBeam();

            if (_isTractorBeamActive && _draggingSecond == null &&
                (gameTime.TotalGameTime - _tractorBeamStartTime) > TimeSpan.FromMilliseconds(1000))
                _isTractorBeamActive = false;
        }

        SpriteBatch _spriteBatch;

        TimeSpan _lastInvincibilityBlinkTime;
        bool _invinsibilityBlinkState;
        bool _invinsible;

        public override void Draw(GameTime gameTime)
        {
            base.Draw(gameTime);

            _spriteBatch.Begin(SpriteSortMode.Deferred,
                BlendState.AlphaBlend,
                SamplerState.PointWrap,
                DepthStencilState.None, RasterizerState.CullCounterClockwise, null);

            if (!Destroying)
            {
                if (!_invinsible || _invinsibilityBlinkState)
                {
                    _spriteBatch.Draw(_texture, ViewportPosition, null, Color.White, RotationAngle, RotationOrigin, Scale,
                        SpriteEffects.None, 0);

                    if (_isTractorBeamActive)
                    {
                        _spriteBatch.Draw(_laserTexture, new Vector2(ViewportPosition.X, ViewportPosition.Y),
                            new Rectangle(0, 0, _laserTexture.Width, (int)ViewportPosition.Y + (int)Size.Y),
                            Color.White, RotationAngle - MathHelper.Pi, Vector2.Zero, 1, SpriteEffects.None, 0);
                    }
                }
            }

            _spriteBatch.End();
        }

        Vector2 _velocityPolar;

        public override Vector2 VelocityPolar
        {
            get
            {
                return _velocityPolar + Level.ViewportVelocityPolar;
            }

            set
            {
                _velocityPolar = value;
            }
        }

        bool _isTractorBeamActive;

        void UpdateTractorBeam()
        {
            if (!_isTractorBeamActive)
                return;

            if (_draggingSecond != null)
            {
                return;
            }

            var sight = GetSightVector();
            float distance;
            var hit = Level.CheckForRayIntersections(this, sight, Level.Seconds, out distance);

            if (hit != null)
            {
                _draggingSecond = (Second)hit;

                _draggingSecond.IsDragged = true;
            }

        }

        public bool IsTractorBeamActive
        {
            get
            {
                return _isTractorBeamActive;
            }

            set
            {
                _isTractorBeamActive = value;
                _draggingSecond = null;

                if (!_isTractorBeamActive)
                {
                    return;
                }

                _tractorBeamSound.Play();

                UpdateTractorBeam();
            }
        }



        TimeSpan _tractorBeamStartTime;

        Second _draggingSecond;

    }
}
