﻿#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Audio;
#endregion

namespace LD27
{
    public class Starfield : DrawableGameComponent
    {
        public Starfield(Level level) : base(level.Game)
        {
            Level = level;
        }

        public Level Level { get; private set; }

        Texture2D _starsTexture;

        protected override void LoadContent()
        {
            base.LoadContent();

            _starsTexture = Game.Content.Load<Texture2D>("Stars");

            _frameWidth = 6;
            _frameCount = _starsTexture.Width / _frameWidth;

            var viewportSize = Level.ViewportSizeCartesian;

            var random = Level.Random;

            for (int i = 0; i < viewportSize.X * viewportSize.Y / _frameWidth / 100; i++)
            {
                var star = new StarInfo
                {
                    FrameIndex = random.Next(_frameCount - 1),
                    Position = new Vector2(random.Next((int)viewportSize.X), random.Next((int)viewportSize.Y)) 
                };

                _stars.Add(star);
            }

            _spriteBatch = new SpriteBatch(GraphicsDevice);
        }

        int _frameCount;
        int _frameWidth;

        List<StarInfo> _stars = new List<StarInfo>();

        class StarInfo
        {
            public int FrameIndex { get; set; }

            public Vector2 Position { get; set; }
        }

        public override void Initialize()
        {
            base.Initialize();
        }

        public override void Update(GameTime gameTime)
        {
            base.Update(gameTime);

        }

        SpriteBatch _spriteBatch;

        public override void Draw(GameTime gameTime)
        {
            base.Draw(gameTime);

            _spriteBatch.Begin(SpriteSortMode.Deferred,
                BlendState.AlphaBlend,
                SamplerState.PointWrap,
                DepthStencilState.None, RasterizerState.CullCounterClockwise, null);


            foreach (var star in _stars)
            {
                _spriteBatch.Draw(_starsTexture, star.Position,
                    new Rectangle(star.FrameIndex * _frameWidth, 0, _frameWidth, _starsTexture.Height),
                    Color.White, 0, Vector2.Zero, 1,
                    SpriteEffects.None, 0);
            }

            _spriteBatch.End();
        }


    }
}
